# -*- coding: utf-8 -*-
"""Video_Analysis_of_Animal_Behaviour.ipynb

Automatically generated by Colab.

Original file is located at
    https://colab.research.google.com/drive/1gpZxSX1zzh4P0k07q0T-2s7cCsbRGo-S
"""

# 1. Install required libraries
!pip install -U -q google-generativeai

# 2. Import libraries
from google.colab import userdata
import google.generativeai as genai
import pandas as pd
import re
import time
import zipfile
import os

# 3. Configure the API
api_key = userdata.get("GOOGLE_API_KEY")
genai.configure(api_key=api_key)

# 4. Unzip the archive
zip_path = "/content/animals.zip"
extract_dir = "/content/animal_videos"
os.makedirs(extract_dir, exist_ok=True)

with zipfile.ZipFile(zip_path, 'r') as zip_ref:
    zip_ref.extractall(extract_dir)

# 5. Define prompts
SYSTEM_PROMPT = "You are a professional ethologist analysing animal behaviour in video footage."
USER_PROMPT = """Please watch the video and provide a single summary description:
1. Identify the animal species (common name + scientific genus).
2. Describe the observed behaviour using appropriate ethological terms (e.g. locomotion, foraging, resting).
Return only one row in a table with the columns: [Animal (Common), Genus, Behaviour Description]."""

# 6. Instantiate the model
model = genai.GenerativeModel(model_name="gemini-1.5-pro")

# 7. Process videos one by one, skipping duplicates
results = []
processed_files = set()  # Keep track of processed filenames

video_files = sorted([
    f for f in os.listdir(extract_dir)
    if f.lower().endswith((".mp4", ".mov", ".avi", ".mkv"))
])

for i, filename in enumerate(video_files, start=1):
    if filename in processed_files:
        print(f" Skipping already processed file: {filename}")
        continue

    file_path = os.path.join(extract_dir, filename)
    print(f"\n Processing file {i}: {filename}")
    processed_files.add(filename)

    try:
        # Upload file
        uploaded_file = genai.upload_file(file_path)
        print(f" Uploaded: {uploaded_file.uri}")

        # Wait for ACTIVE status
        for attempt in range(10):
            file_status = genai.get_file(uploaded_file.name)
            if file_status.state.name == "ACTIVE":
                print(" File is ACTIVE.")
                break
            else:
                print(f" Waiting for file to become ACTIVE... (Attempt {attempt+1})")
                time.sleep(2)
        else:
            print(f" Skipping {filename} – file not ACTIVE in time.")
            continue

        # Generate response
        response = model.generate_content(
            contents=[
                {
                    "role": "user",
                    "parts": [
                        {
                            "file_data": {
                                "mime_type": file_status.mime_type,
                                "file_uri": file_status.uri
                            }
                        },
                        {
                            "text": SYSTEM_PROMPT + "\n\n" + USER_PROMPT
                        }
                    ]
                }
            ],
            generation_config={
                "temperature": 0.2,
                "max_output_tokens": 1000
            }
        )

        # Extract markdown table
        lines = response.text.strip().split('\n')
        table_lines = [line for line in lines if line.strip().startswith('|')]

        headers = [h.strip() for h in table_lines[0].strip('|').split('|')]
        row = [cell.strip() for cell in table_lines[2].strip('|').split('|')]

        results.append([i] + row)

    except Exception as e:
        print(f"Error processing {filename}: {e}")

# 8. Export results to Excel
if results:
    df = pd.DataFrame(results, columns=["ID", "Animal (Common)", "Genus", "Behaviour Description"])
    output_path = "/content/animal_behaviour_summary.xlsx"
    df.to_excel(output_path, index=False)
    print(f"\n Exported all results to: {output_path}")
else:
    print("No data to export.")